import json
import os.path
import numpy as np
import matplotlib.pyplot as plt
from yupi.tracking.affine_estimator import affine_matrix


def affine2camera(theta, tx, ty):
    x_cl, y_cl, theta_cl = np.zeros((3, theta.size + 1))
    theta_cl[1:] = np.cumsum(theta)

    for i in range(theta.size):
        A = affine_matrix(theta_cl[i + 1], x_cl[i], y_cl[i], R_inv=True)
        x_cl[i + 1], y_cl[i + 1] = A @ [-tx[i], -ty[i], 1]

    x_cl, y_cl, theta_cl = x_cl[1:], y_cl[1:], theta_cl[1:]
    return x_cl, y_cl, theta_cl


def camera2ant(x_ac, y_ac, x_cl, y_cl, theta_cl):
    x_al, y_al = np.empty((2, x_ac.size))

    for i in range(x_ac.size):
        A = affine_matrix(theta_cl[i], x_cl[i], y_cl[i], R_inv=True)
        x_al[i], y_al[i] = A @ [x_ac[i], y_ac[i], 1]

    return x_al, y_al


def affine2ant(theta, tx, ty, x_ac, y_ac):
    x_cl, y_cl, theta_cl = affine2camera(theta, tx, ty)
    x_al, y_al = camera2ant(x_ac, y_ac, x_cl, y_cl, theta_cl)
    return x_al, y_al



def plot_results(file_dir):
    pix_per_cm = 63.
    pix_per_m = pix_per_cm * 100

    # file_name = 'data_video2_[1.2min-100.0%].json'
    # file_dir = os.path.join('data', file_name)
    with open(file_dir, 'r') as file:
        data = json.load(file)

    # get affine parameters and ant coordinates in the camera frame of reference
    affine_params = np.array(data['affine_params'])
    r_ac = np.array(data['r_ac'], dtype=np.float)

    # unpack and convert from pixels to meters
    theta, tx, ty, _ = affine_params.T
    tx, ty = tx / pix_per_m, ty / pix_per_m
    x_ac, y_ac = r_ac.T / pix_per_m

    # invert axis
    theta *= -1
    ty *= -1
    y_ac *= -1

    # ant position with respect to the lab
    x_al, y_al = affine2ant(theta, tx, ty, x_ac, y_ac)
    x = x_al - x_al[0]
    y = y_al - y_al[0]


    # plotting
    plt.plot(x, y, '-')
    plt.plot(x[0], y[0], 'go', mfc='white', zorder=2)
    plt.plot(x[-1], y[-1], 'ro', mfc='white', zorder=2)
    plt.plot(x[0], y[0], 'go', mec='g', alpha=.5, label='initial position')
    plt.plot(x[-1], y[-1], 'ro', mec='r', alpha=.5, label='final position')
    plt.legend(fontsize=12)

    plt.title('ant position with respect to the lab')
    plt.tick_params(direction='in', labelsize=12)
    plt.axis('equal')
    plt.grid(True)
    plt.xlabel('x [m]', fontsize=12)
    plt.ylabel('y [m]', fontsize=12)

    plt.show()


import os
import cv2
import numpy as np
import abc

class Undistorter(metaclass=abc.ABCMeta):

    """Undistorts images using camera calibration matrix"""

    def __init__(self, camera_file):

        # Read camera undistort matrix
        npzfile = np.load(camera_file)

        # Initialize camera parameters
        self.c_h = npzfile['h']
        self.c_w = npzfile['w']
        self.c_mtx = npzfile['mtx']
        self.c_dist = npzfile['dist']
        self.c_newcameramtx = npzfile['newcameramtx']
        self.c_mapx, self.c_mapy = cv2.initUndistortRectifyMap(self.c_mtx, self.c_dist, 
                                                               None, self.c_newcameramtx,
                                                               (self.c_w, self.c_h), 5)
        self.mask = None

    @abc.abstractmethod
    def undistort(self, frame):
        pass

    # method to be called to fix the distortion
    def fix(self, frame):
        if self.mask is None:
            self.create_mask(frame)
        corrected = self.undistort(frame)
        return self.masked(corrected)

    # create a mask with the distortion pattern
    def create_mask(self, frame):
        empty_frame = 255 * np.ones(frame.shape, dtype=np.uint8)        
        corrected = self.undistort(empty_frame)
        kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE,(5,5))
        self.mask = cv2.erode(corrected, kernel, iterations=1)
        self.mask = cv2.bitwise_not(self.mask)
        self.background = np.full(frame.shape, 130, dtype=np.uint8)

    # apply the mask to a frame to adjust border colors
    def masked(self, frame):
        return cv2.bitwise_or(frame, self.mask)


class ClassicUndistorter(Undistorter):
    """docstring for ClassicUndistorter"""
    def __init__(self, camera_file):
        super(ClassicUndistorter, self).__init__(camera_file)

    def undistort(self, frame):
        return cv2.undistort(frame, self.c_mtx, self.c_dist, None, self.c_newcameramtx)  


class RemapUndistorter(Undistorter):
    """docstring for RemapUndistorter"""
    def __init__(self, camera_file):
        super(RemapUndistorter, self).__init__(camera_file)

    def undistort(self, frame):
        return cv2.remap(frame, self.c_mapx, self.c_mapy, cv2.INTER_LINEAR)


class NoUndistorter(Undistorter):
    """docstring for NoUndistorter"""
    def __init__(self, camera_file):
        super(NoUndistorter, self).__init__(camera_file)

    def undistort(self, frame):
        return frame



from tracking.trackers import ROI, ObjectTracker, CameraTracker, TrackingScenario
from tracking.undistorters import RemapUndistorter
from tracking.show_path import plot_results
from tracking.algorithms import IntensityMatching

if __name__ == '__main__':
    # Initialize main tracking objects
    algorithm = IntensityMatching(20, 180, 180)
    ant = ObjectTracker('ant', algorithm, ROI((120, 120)))
    camera = CameraTracker(ROI((.65, .65), ROI.CENTER_INIT_MODE))
    undistorter = RemapUndistorter('cameras/gph3+1080-60fps-NARROW.npz')
    scenario = TrackingScenario([ant], camera, undistorter)

    # Track the video using the preconfigured scenario
    retval, message = scenario.track('videos/video2_short.mp4')
    print(message)
    # plot_results('videos/video2_short_[0.2min-100.0%].json')


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yupi', 'yupi.tracking']

package_data = \
{'': ['*'], 'yupi': ['cameras/*']}

install_requires = \
['PyQt5>=5.15.3,<6.0.0',
 'matplotlib>=3.2.0',
 'nudged>=0.3.1',
 'numpy>=1.16.5',
 'opencv-python>=4.4.0']

setup_kwargs = {
    'name': 'yupi',
    'version': '0.0.1',
    'description': 'A package for tracking and analysing objects trajectories',
    'long_description': '# Yupi\n\nLong description of the package\n',
    'author': 'Gustavo Viera-López',
    'author_email': 'gvieralopez@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
